; ==============================================================================
; file: "source/system/kernel.asm"
; use: to be compiled into "system/kernel.bin"
; ==============================================================================

use32
org 20000h

glaux_define_calls:                     ; NEVER CHANGE!

; TODO: check all calls and fill in the gaps

    .entry:
        jmp kernel                      ; 00020000h
        
    .display:
        jmp display_fill                ; 00020005h
        jmp display_update              ; 0002000Ah
        jmp display_paste_str           ; 0002000Fh
        jmp display_paste_rect          ; 00020014h
        jmp display_paste_char          ; 00020019h
        jmp display_scroll              ; 0002001Eh
        times 256 - ($ - .entry) db 0

    .string:
        jmp string_compare              ; 00020100h
        times 256 - ($ - .string) db 0

    .keyboard:
        jmp keybd_get_scan              ; 00020200h
        jmp keybd_conv_latin            ; 00020205h
        jmp keybd_input_str             ; 0002020Ah
        times 512 - ($ - .keyboard) db 0
    
    .ehci:
        ; it is dangerous to have these calls accessible to user programs
    
    .pci:
        ; it is dangerous to have these calls accessible to user programs
    
    .math:
        jmp math_int_to_hex             ; 00020400h
        times 256 - ($ - .math) db 0
    
    .ext2:
        jmp ext2_read_file              ; 00020500h
        times 256 - ($ - .ext2) db 0

; ==============================================================================

glaux_include_calls_vars:
    %include "source/system/kernel/display.inc" ; display calls
    %include "source/system/kernel/string.inc"  ; string calls
    %include "source/system/kernel/keybd.inc"   ; keyboard calls
    %include "source/system/kernel/ehci.inc"    ; ehci calls
    %include "source/system/kernel/pci.inc"     ; pci calls
    %include "source/system/kernel/math.inc"    ; math calls
    %include "source/system/kernel/ext2.inc"    ; ext2 calls
    %include "source/variables.inc"             ; variables used for above calls

; ------------------------------------------------------------------------------

hidden_ehci_variables:

    ; it is dangerous to have these variables accessible to user programs

    ehci_bus               equ 0x00012000
    ehci_dev               equ 0x00012004
    ehci_fun               equ 0x00012008
    ehci_port_number       equ 0x0001200C
    ehci_base              equ 0x00012010
    ehci_opbase            equ 0x00012014
    ehci_eecp              equ 0x00012018
    ehci_TD_endpoint       equ 0x00012020
    ehci_TD_bus_address    equ 0x00012024
    ehci_TD_buffer_size    equ 0x00012028
    ehci_TD_buffer_pointer equ 0x0001202C
    ehci_TD_packet_id      equ 0x00012030
    ehci_TD_toggle_pointer equ 0x00012034
    ehci_control_toggle    equ 0x00012040
    ehci_in_toggle         equ 0x00012044
    ehci_out_toggle        equ 0x00012048
    ehci_bus_address       equ 0x00012050
    ehci_in_endpoint       equ 0x00012054
    ehci_out_endpoint      equ 0x00012058
    
    ehci_control_QH        equ 0x0001F000
    ehci_device_descriptor equ 0x0001F100
    ehci_config_descriptor equ 0x0001F200
    ehci_sense             equ 0x0001F300
    ehci_inquiry_str       equ 0x0001F400
    ehci_capacity          equ 0x0001F500
    ehci_block_size        equ 0x0001F504
    ehci_scsi_csw          equ 0x0001F600
    
    ehci_TD_chain          equ 0x00C00000

; ------------------------------------------------------------------------------

hidden_pci_variables:

    ; it is dangerous to have these variables accessible to user programs
    
    pci_address            equ 0x00013000
    pci_offset             equ 0x00013004
    pci_data               equ 0x00013008
    pci_bus                equ 0x00013020
    pci_dev                equ 0x00013024
    pci_fun                equ 0x00013028

; ==============================================================================

kernel:
    
    .clear_display:
        mov dword [color_1], 0          ; we make screen black
        mov dword [color_2], 0xCCFF99   ; and we prepare green letters
        call display_fill
        call display_update

    .init_ehci_ctrller:
        call ehci_init_ctrller          ; we init the ehci ctrller
        call ehci_init_dev0             ; and the boot device
    
    .load_font:
        mov dword [string_1], font_path         ; we load the default font
        mov dword [ram_address], font_buffer    ; at 0x00200000
        call ext2_read_file
    
    .load_keybd_layout:
        mov dword [string_1], layout_path       ; we load the keyboard layout
        mov dword [ram_address], keybd_layout   ; at 0x00100000
        call ext2_read_file
        
    .go_to_start_of_screen:
        mov dword [pixel_horiz], 0      ; we need to go to the top left border
        mov dword [pixel_vert], 0       ; so the prompt starts properly
        
    .get_next_str:
        mov dword [string_1], prompt    ; we paste prompt string
        call display_paste_str
        call display_update
        mov dword [string_1], cmd       ; we get string from keyboard
        call keybd_input_str
    
    .add_line:
        mov dword [pixel_horiz], 0      ; we go to left border
        add dword [pixel_vert], 16      ; and 16 pixels down
        cmp dword [pixel_vert], 480     ; if bottom border of screen
        jb .get_next_str                ; we need to scroll
    
    .scroll:
        call display_scroll
        call display_update
        jmp .get_next_str

prompt      db  ">",0," ",0,0,0                         ; prompt for terminal
font_path   db "/dev0/system/fonts/glaux-mono.sys",0    ; ascii path for font
layout_path db "/dev0/system/layout.sys",0              ; ascii path for layout

times 12288 - ($ - $$) db 0             ; pad with zeros until 12 kilobytes

cmd:                                    ; we store the command after the kernel
