; ==============================================================================
; file: "source/system/kernel/ehci.inc"
; use: to be included in "source/system/kernel.asm"
; ==============================================================================

ehci_find_ctrller:      

    .return:
        ret

; ==============================================================================

ehci_init_ctrller:

    .set_ehci_address:
        mov eax, dword [ehci_bus]
        mov dword [pci_bus], eax
        mov eax, dword [ehci_dev]
        mov dword [pci_dev], eax
        mov eax, dword [ehci_fun]
        mov dword [pci_fun], eax
        call pci_set_config_address
        ; [pci_address] = ehci bus:dev:fun enabled

    .enable_bus_master_and_memory_space:
        mov dword [pci_offset], 04h
        call pci_read_dword
        or dword [pci_data], 06h
        call pci_write_dword
    
    .get_base_address:
        mov dword [pci_offset], 10h
        call pci_read_dword
        mov eax, dword [pci_data]
        and eax, 0FFFFFFF0h ; 0FFFFFF00h
        mov dword [ehci_base], eax
    
    .get_opbase_address:
        mov esi, dword [ehci_base]       ; get byte at ehci base address
        movzx eax, byte [esi]
        add eax, esi                     ; add ehci base address to it
        mov dword [ehci_opbase], eax     ; save ehci operation base address
    
    .get_eecp_offset:
        movzx eax, word [esi + 8]
        shr eax, 8
        mov dword [ehci_eecp], eax
    
    .check_valid_eecp:
        cmp eax, 40h
        jb .stop_ctrller
    
    .check_bios_control:
        mov dword [pci_offset], eax ; read from eecp
        call pci_read_dword
        test dword [pci_data], 10000h
        jz .stop_ctrller
    
    .get_control:
        or dword [pci_data], 1000000h
        call pci_write_dword

    .pause_1:
        mov ecx, 08000000h
    
    .pause_2:
        sub ecx, 1
        jnz .pause_2
    
    ; check again for halted ctrller
    
    .stop_ctrller:
        mov esi, dword [ehci_opbase]
        and dword [esi], 0FFFFFFFEh
    
    .check_halted:
        test dword [esi + 4], 1000h
        jz .check_halted
    
    .reset_ctrller:
        mov esi, dword [ehci_opbase]
        or dword [esi], 02h
    
    .check_reset:
        test dword [esi], 02h
        jnz .check_reset
    
    .disable_ints:
        mov esi, dword [ehci_opbase]
        mov dword [esi + 8], 0
    
    .clear_control_data_struct_segment:
        mov dword [esi + 16], 0
    
    .set_QH_link_pointer:
        mov dword [ehci_control_QH], ehci_control_QH
        or dword [ehci_control_QH], 2
    
    .set_QH_endpoint_characteristics:
        mov dword [ehci_control_QH + 4], 5200E000h ; 5200C000h / 0200E000h
    
    .set_QH_endpoint_capabilities:
        mov dword [ehci_control_QH + 8], 40000000h
    
    .set_current_TD_pointer:
        mov dword [ehci_control_QH + 12], 0 ; written actually by controller
    
    .set_TD_overlay_area:
        mov dword [ehci_control_QH + 16], 1
        mov dword [ehci_control_QH + 20], 0
        mov dword [ehci_control_QH + 24], 0
        mov dword [ehci_control_QH + 28], 0
        mov dword [ehci_control_QH + 32], 0
        mov dword [ehci_control_QH + 36], 0
        mov dword [ehci_control_QH + 40], 0
        mov dword [ehci_control_QH + 44], 0
        
        mov dword [ehci_control_QH + 48], 0
        mov dword [ehci_control_QH + 52], 0
        mov dword [ehci_control_QH + 56], 0
        mov dword [ehci_control_QH + 60], 0
        mov dword [ehci_control_QH + 64], 0
    
    .save_address_of_QH:
        mov esi, dword [ehci_opbase]
        mov dword [esi + 24], ehci_control_QH
    
    .set_config_flag:
        or dword [esi + 64], 1
        
    .pause_3:
        mov ecx, 10000000h
    
    .pause_4:
        sub ecx, 1
        jnz .pause_4
    
    .enable_async_list:
        mov esi, dword [ehci_opbase]
        or dword [esi], 20h
    
    .start_ctrller:
        or dword [esi], 1
    
    .pause_5:
        mov ecx, 10000000h
    
    .pause_6:
        sub ecx, 1
        jnz .pause_6
    
    .clear_toggle_vars:
        mov dword [ehci_in_toggle], 0
        mov dword [ehci_out_toggle], 0
        mov dword [ehci_control_toggle], 0

    .return:
        ret

; ==============================================================================

ehci_make_run_TD:

    .prepare:
        mov esi, ehci_TD_chain
        mov edi, ehci_TD_chain + 64
        mov eax, dword [ehci_TD_buffer_size]
        mov dword [.remaining], eax

    .modify_QH_endpoint:
        mov eax, dword [ehci_control_QH + 4]
        and eax, 0FFFFF0FFh
        mov ebx, dword [ehci_TD_endpoint]
        shl ebx, 8
        or eax, ebx
    
    .modify_QH_address:
        and eax, 0FFFFFF80h
        or eax, dword [ehci_TD_bus_address]
        mov dword [ehci_control_QH + 4], eax
    
    ; ----------------------
    ; first and second dword
    ; ----------------------
    
    .start_loop:
        cmp dword [.remaining], 512
        jbe .terminate_chain
    
    .link_to_next_TD:
        mov dword [esi], edi
        mov dword [esi + 4], 1
        jmp .get_toggle
    
    .terminate_chain:
        mov dword [esi], 1
        mov dword [esi + 4], 1
    
    ; -----------
    ; third dword
    ; -----------
    
    .get_toggle:
        mov ebx, dword [ehci_TD_toggle_pointer]
        mov eax, dword [ebx]
        shl eax, 31
        xor dword [ebx], 1
    
    .get_bytes_to_transfer:
        cmp dword [.remaining], 512
        jbe .set_remaining
    
    .set_512:
        mov ebx, 512
        shl ebx, 16
        or eax, ebx
        jmp .set_error_counter
    
    .set_remaining:
        mov ebx, dword [.remaining]
        shl ebx, 16
        or eax, ebx
    
    .set_error_counter:
        or eax, 0C00h
    
    .get_packet_id:
        mov ebx, dword [ehci_TD_packet_id]
        shl ebx, 8
        or eax, ebx
    
    .set_active_status:
        or eax, 80h
    
    .write_third_dword_to_TD:
        mov dword [esi + 8], eax
    
    ; -----------
    ; next dwords
    ; -----------
    
    .set_buffer_pointer_pages:
        mov eax, dword [ehci_TD_buffer_pointer]
        mov dword [esi + 12], eax
        mov dword [esi + 16], 0
        mov dword [esi + 20], 0
        mov dword [esi + 24], 0
        mov dword [esi + 28], 0
        ; and high 32 bits
        mov dword [esi + 32], 0
        mov dword [esi + 36], 0
        mov dword [esi + 40], 0
        mov dword [esi + 44], 0
        mov dword [esi + 48], 0
    
    .compare_end:
        cmp dword [.remaining], 512
        jbe .run_TD
    
    .go_to_next_TD:
        add esi, 64
        add edi, 64
        sub dword [.remaining], 512
        add dword [ehci_TD_buffer_pointer], 512
        jmp .start_loop
    
    .run_TD:
        mov dword [ehci_control_QH + 16], ehci_TD_chain
    
    .pause_1:
        mov ecx, 04000000h
    
    .pause_2:
        sub ecx, 1
        jz .return
    
    .check_for_halted:
        test dword [esi + 8], 40h
        jnz .pause_2
    
    .check_for_inactive:
        test dword [esi + 8], 80h
        jnz .pause_2
    
    .return:
        ret
    
    .remaining dd 0

; ==============================================================================

ehci_init_dev0:
    
    .set_port_0:
        mov dword [ehci_port_number], 0
    
    .check_port_connection:
        call ehci_check_connected_port
        jz .go_to_next_port
    
    .check_low_speed:
        call ehci_check_low_speed_port
        je .go_to_next_port
    
    .reset_port:
        call ehci_reset_port
    
    .get_device_descriptor:
        call ehci_get_device_descriptor
        call ehci_get_device_descriptor
    
    .get_config_descriptor:
        mov dword [ehci_TD_buffer_size], 9
        call ehci_get_config_descriptor
    
    .get_remaining_descriptors:
        movzx eax, byte [ehci_config_descriptor + 2]
        mov dword [ehci_TD_buffer_size], eax
        call ehci_get_config_descriptor
    
    .check_mass_storage:
        cmp byte [ehci_config_descriptor + 14], 08h
        jne .go_to_next_port
    
    .check_scsi:
        cmp byte [ehci_config_descriptor + 15], 06h
        jne .go_to_next_port
    
    .check_bulk_only:
        cmp byte [ehci_config_descriptor + 16], 50h
        jne .go_to_next_port
    
    .get_endpoint_1:
        mov al, byte [ehci_config_descriptor + 20]
        test al, 80h
        jnz .save_endpoint_in_1
    
    .save_endpoint_out_1:
        mov byte [ehci_out_endpoint], al
        and dword [ehci_out_endpoint], 0Fh
        jmp .get_endpoint_2
    
    .save_endpoint_in_1:
        mov byte [ehci_in_endpoint], al
        and dword [ehci_in_endpoint], 0Fh
    
    .get_endpoint_2:
        mov al, byte [ehci_config_descriptor + 27]
        test al, 80h
        jnz .save_endpoint_in_2
        
    .save_endpoint_out_2:
        mov byte [ehci_out_endpoint], al
        and dword [ehci_out_endpoint], 0Fh
        ; jmp .return
        jmp .set_address
    
    .save_endpoint_in_2:
        mov byte [ehci_in_endpoint], al
        and dword [ehci_in_endpoint], 0Fh
    
    ; until here ok
    
    .set_address:
        mov dword [ehci_bus_address], 1
        call ehci_set_address
    
    .set_config:
        call ehci_set_config
    
    .reset_dev0_toggle:
        mov dword [ehci_in_toggle], 0
        mov dword [ehci_out_toggle], 0
    
        call ehci_inquiry
        call ehci_test_unit
        call ehci_request_sense
        call ehci_test_unit
        call ehci_request_sense
        
        call ehci_read_capacity
        mov eax, dword [ehci_capacity]
        bswap eax
        mov dword [ehci_capacity], eax
        mov eax, dword [ehci_block_size]
        bswap eax
        mov dword [ehci_block_size], eax
        
        jmp .return
    
    .go_to_next_port:
        add dword [ehci_port_number], 1
        cmp dword [ehci_port_number], 4
        jb .check_port_connection
    
    .return:
        ret

; ==============================================================================

ehci_check_connected_port:

    .get_port_number:
        mov eax, dword [ehci_port_number]
    
    .check_connected_port:
        mov esi, dword [ehci_opbase]
        test dword [esi + 44h + eax*4], 1

    .return:
        ret

; ==============================================================================

ehci_check_low_speed_port:
    
    .get_port_number:
        mov eax, dword [ehci_port_number]
    
    .check_low_speed:
        mov esi, dword [ehci_opbase]
        mov ebx, dword [esi + 44h + eax*4]
        shr ebx, 10
        and ebx, 3
        cmp ebx, 1
    
    .return:
        ret
        
; ==============================================================================

ehci_reset_port:
    
    .get_port_number:
        mov eax, dword [ehci_port_number]
    
    .send_reset_signal:
        mov esi, dword [ehci_opbase]
        mov ebx, dword [esi + 44h + eax*4]
        or ebx, 100h
        and ebx, 0FFFFFFFBh
        mov dword [esi + 44h + eax*4], ebx
    
    .pause_1:
        mov ecx, 04000000h
    
    .pause_2:
        sub ecx, 1
        jnz .pause_2
    
    .stop_reset_signal:
        and dword [esi + 44h + eax*4], 0FFFFFEFFh
    
    .pause_3:
        mov ecx, 04000000h
    
    .check_reset:
        test dword [esi + 44h + eax*4], 100h
        jz .return
    
    .pause_4:
        sub ecx, 1
        jnz .check_reset
    
    .return:
        cmp dword [esi + 44h + eax*4], 1005h
        ret

; ==============================================================================

ehci_get_device_descriptor:

    .transport_command:
        mov dword [ehci_TD_buffer_pointer], .request
        mov dword [ehci_TD_buffer_size], 8
        mov dword [ehci_TD_packet_id], 2
        mov dword [ehci_control_toggle], 0
        mov dword [ehci_TD_toggle_pointer], ehci_control_toggle
        mov dword [ehci_TD_endpoint], 0
        mov dword [ehci_TD_bus_address], 0
        call ehci_make_run_TD
    
    .transport_data:
        mov dword [ehci_TD_buffer_pointer], ehci_device_descriptor
        mov dword [ehci_TD_buffer_size], 18
        mov dword [ehci_TD_packet_id], 1
        mov dword [ehci_control_toggle], 1
        mov dword [ehci_TD_toggle_pointer], ehci_control_toggle
        mov dword [ehci_TD_endpoint], 0
        mov dword [ehci_TD_bus_address], 0
        call ehci_make_run_TD
    
    .return:
        ret

    .request:
        db 80h
        db 6
        dw 100h
        dw 0
        dw 18

; ==============================================================================

ehci_get_config_descriptor:
        
        mov eax, dword [ehci_TD_buffer_size]
        mov dword [.total_length], eax
        mov word [.request + 6], ax

    .transport_command:
        mov dword [ehci_TD_buffer_pointer], .request
        mov dword [ehci_TD_buffer_size], 8
        mov dword [ehci_TD_packet_id], 2
        mov dword [ehci_control_toggle], 0
        mov dword [ehci_TD_toggle_pointer], ehci_control_toggle
        mov dword [ehci_TD_endpoint], 0
        mov dword [ehci_TD_bus_address], 0
        call ehci_make_run_TD
    
    .transport_data:
        mov dword [ehci_TD_buffer_pointer], ehci_config_descriptor
        mov eax, dword [.total_length]
        mov dword [ehci_TD_buffer_size], eax
        mov dword [ehci_TD_packet_id], 1
        mov dword [ehci_control_toggle], 1
        mov dword [ehci_TD_toggle_pointer], ehci_control_toggle
        mov dword [ehci_TD_endpoint], 0
        mov dword [ehci_TD_bus_address], 0
        call ehci_make_run_TD
    
    .return:
        ret
    
    .total_length dd 0

    .request:
        db 80h
        db 6
        dw 200h
        dw 0
        dw 9

; ==============================================================================

ehci_set_address:
    
    .prepare:
        mov ax, word [ehci_bus_address]
        mov word [.request + 2], ax

    .transport_command:
        mov dword [ehci_TD_buffer_pointer], .request
        mov dword [ehci_TD_buffer_size], 8
        mov dword [ehci_TD_packet_id], 2
        mov dword [ehci_control_toggle], 0
        mov dword [ehci_TD_toggle_pointer], ehci_control_toggle
        mov dword [ehci_TD_endpoint], 0
        mov dword [ehci_TD_bus_address], 0
        call ehci_make_run_TD
    
    .transport_status:
        mov dword [ehci_TD_buffer_pointer], 0
        mov dword [ehci_TD_buffer_size], 0
        mov dword [ehci_TD_packet_id], 1
        mov dword [ehci_control_toggle], 1
        mov dword [ehci_TD_toggle_pointer], ehci_control_toggle
        mov dword [ehci_TD_endpoint], 0
        mov dword [ehci_TD_bus_address], 0
        call ehci_make_run_TD
        
    .return:
        movzx eax, word [ehci_bus_address]
        mov dword [ehci_TD_bus_address], eax
        ret

    .request:
        db 0
        db 5
        dw 0
        dw 0
        dw 0

; ==============================================================================

ehci_set_config:
    
    .prepare:
        mov al, byte [ehci_config_descriptor + 5]
        mov byte [.request + 2], al

    .transport_command:
        mov dword [ehci_TD_buffer_pointer], .request
        mov dword [ehci_TD_buffer_size], 8
        mov dword [ehci_TD_packet_id], 2
        mov dword [ehci_control_toggle], 0
        mov dword [ehci_TD_toggle_pointer], ehci_control_toggle
        mov dword [ehci_TD_endpoint], 0
        ; ehci_TD_bus_address already set
        call ehci_make_run_TD
        
    .transport_status:
        mov dword [ehci_TD_buffer_pointer], 0
        mov dword [ehci_TD_buffer_size], 0
        mov dword [ehci_TD_packet_id], 1
        mov dword [ehci_control_toggle], 1
        mov dword [ehci_TD_toggle_pointer], ehci_control_toggle
        mov dword [ehci_TD_endpoint], 0
        ; ehci_TD_bus_address already set
        call ehci_make_run_TD
    
    .return:
        ret

    .request:
        db 0
        db 9
        dw 1
        dw 0
        dw 0

; ==============================================================================
 
ehci_inquiry:

    .transport_command:
        mov dword [ehci_TD_buffer_pointer], .request
        mov dword [ehci_TD_buffer_size], 31
        mov dword [ehci_TD_packet_id], 0
        mov dword [ehci_TD_toggle_pointer], ehci_out_toggle
        mov eax, dword [ehci_out_endpoint]
        mov dword [ehci_TD_endpoint], eax
        call ehci_make_run_TD
    
    .pause_1:
        mov ecx, 01000000h
    
    .pause_2:
        sub ecx, 1
        jnz .pause_2
    
    .transport_data:
        mov dword [ehci_TD_buffer_pointer], ehci_inquiry_str
        mov dword [ehci_TD_buffer_size], 36
        mov dword [ehci_TD_packet_id], 1
        mov dword [ehci_TD_toggle_pointer], ehci_in_toggle
        mov eax, dword [ehci_in_endpoint]
        mov dword [ehci_TD_endpoint], eax
        call ehci_make_run_TD
    
    .pause_3:
        mov ecx, 01000000h
    
    .pause_4:
        sub ecx, 1
        jnz .pause_4
    
    .transport_status:
        mov dword [ehci_TD_buffer_pointer], ehci_scsi_csw
        mov dword [ehci_TD_buffer_size], 13
        mov dword [ehci_TD_packet_id], 1
        mov dword [ehci_TD_toggle_pointer], ehci_in_toggle
        mov eax, dword [ehci_in_endpoint]
        mov dword [ehci_TD_endpoint], eax
        call ehci_make_run_TD
    
    .pause_5:
        mov ecx, 01000000h
    
    .pause_6:
        sub ecx, 1
        jnz .pause_6
    
    .return:
        ret

    .request:
        dd 43425355h
        dd 0AABBCCDDh
        dd 36
        db 80h
        db 0
        db 6
        db 12h
        db 0
        db 0
        db 0
        db 36
        db 0
        times 15 db 0
        ;times 10 db 0

; ==============================================================================

ehci_request_sense:

    .transport_command:
        mov dword [ehci_TD_buffer_pointer], .request
        mov dword [ehci_TD_buffer_size], 31
        mov dword [ehci_TD_packet_id], 0
        mov dword [ehci_TD_toggle_pointer], ehci_out_toggle
        mov eax, dword [ehci_out_endpoint]
        mov dword [ehci_TD_endpoint], eax
        call ehci_make_run_TD
    
    .pause_1:
        mov ecx, 01000000h
    
    .pause_2:
        sub ecx, 1
        jnz .pause_2
    
    .transport_data:
        mov dword [ehci_TD_buffer_pointer], ehci_sense
        mov dword [ehci_TD_buffer_size], 18
        mov dword [ehci_TD_packet_id], 1
        mov dword [ehci_TD_toggle_pointer], ehci_in_toggle
        mov eax, dword [ehci_in_endpoint]
        mov dword [ehci_TD_endpoint], eax
        call ehci_make_run_TD
    
    .pause_3:
        mov ecx, 01000000h
    
    .pause_4:
        sub ecx, 1
        jnz .pause_4
    
    .transport_status:
        mov dword [ehci_TD_buffer_pointer], ehci_scsi_csw
        mov dword [ehci_TD_buffer_size], 13
        mov dword [ehci_TD_packet_id], 1
        mov dword [ehci_TD_toggle_pointer], ehci_in_toggle
        mov eax, dword [ehci_in_endpoint]
        mov dword [ehci_TD_endpoint], eax
        call ehci_make_run_TD
    
    .pause_5:
        mov ecx, 01000000h
    
    .pause_6:
        sub ecx, 1
        jnz .pause_6
    
    .return:
        ret

    .request:
        dd 43425355h
        dd 0AABBCCDDh
        dd 18
        db 80h
        db 0
        db 6
        db 03h
        db 0
        dd 0
        db 0
        dw 18
        db 0
        times 6 db 0

; ==============================================================================

ehci_test_unit:

    .transport_command:
        mov dword [ehci_TD_buffer_pointer], .request
        mov dword [ehci_TD_buffer_size], 31
        mov dword [ehci_TD_packet_id], 0
        mov dword [ehci_TD_toggle_pointer], ehci_out_toggle
        mov eax, dword [ehci_out_endpoint]
        mov dword [ehci_TD_endpoint], eax
        call ehci_make_run_TD
    
    .pause_1:
        mov ecx, 01000000h
    
    .pause_2:
        sub ecx, 1
        jnz .pause_2
    
    .transport_status:
        mov dword [ehci_TD_buffer_pointer], ehci_scsi_csw
        mov dword [ehci_TD_buffer_size], 13
        mov dword [ehci_TD_packet_id], 1
        mov dword [ehci_TD_toggle_pointer], ehci_in_toggle
        mov eax, dword [ehci_in_endpoint]
        mov dword [ehci_TD_endpoint], eax
        call ehci_make_run_TD
    
    .pause_3:
        mov ecx, 01000000h
    
    .pause_4:
        sub ecx, 1
        jnz .pause_4
    
    .return:
        ret

    .request:
        dd 43425355h
        dd 0AABBCCDDh
        dd 0
        db 0
        db 0
        db 6
        db 0
        dd 0
        db 0
        db 0
        db 0
        db 0
        times 15 db 0

; ==============================================================================

ehci_read_capacity:

    .transport_command:
        mov dword [ehci_TD_buffer_pointer], .request
        mov dword [ehci_TD_buffer_size], 31
        mov dword [ehci_TD_packet_id], 0
        mov dword [ehci_TD_toggle_pointer], ehci_out_toggle
        mov eax, dword [ehci_out_endpoint]
        mov dword [ehci_TD_endpoint], eax
        call ehci_make_run_TD
    
    .pause_1:
        mov ecx, 01000000h
    
    .pause_2:
        sub ecx, 1
        jnz .pause_2
    
    .transport_data:
        mov dword [ehci_TD_buffer_pointer], ehci_capacity
        mov dword [ehci_TD_buffer_size], 8
        mov dword [ehci_TD_packet_id], 1
        mov dword [ehci_TD_toggle_pointer], ehci_in_toggle
        mov eax, dword [ehci_in_endpoint]
        mov dword [ehci_TD_endpoint], eax
        call ehci_make_run_TD

    .pause_3:
        mov ecx, 01000000h
    
    .pause_4:
        sub ecx, 1
        jnz .pause_4

    .transport_status:
        mov dword [ehci_TD_buffer_pointer], ehci_scsi_csw
        mov dword [ehci_TD_buffer_size], 13
        mov dword [ehci_TD_packet_id], 1
        mov dword [ehci_TD_toggle_pointer], ehci_in_toggle
        mov eax, dword [ehci_in_endpoint]
        mov dword [ehci_TD_endpoint], eax
        call ehci_make_run_TD

    .pause_5:
        mov ecx, 01000000h
    
    .pause_6:
        sub ecx, 1
        jnz .pause_6

    .return:
        ret

    .request:
        dd 43425355h
        dd 0AABBCCDDh
        dd 8
        db 80h
        db 0
        db 10
        db 25h
        db 0
        dd 0
        dw 0
        db 0
        db 0
        times 6 db 0

; ==============================================================================

ehci_read_sectors:

        mov eax, dword [sector_count]
        shl eax, 9
        mov dword [.request + 8], eax
        
        mov eax, dword [lba_sector]
        bswap eax
        mov dword [.request + 17], eax
        
        mov eax, dword [sector_count]
        mov byte [.request + 22], ah
        mov byte [.request + 23], al

    .transport_command:
        mov dword [ehci_TD_buffer_pointer], .request
        mov dword [ehci_TD_buffer_size], 31
        mov dword [ehci_TD_packet_id], 0
        mov dword [ehci_TD_toggle_pointer], ehci_out_toggle
        mov eax, dword [ehci_out_endpoint]
        mov dword [ehci_TD_endpoint], eax
        call ehci_make_run_TD
    
    .pause_1:
        mov ecx, 01000000h
    
    .pause_2:
        sub ecx, 1
        jnz .pause_2
    
    .transport_data:
        mov eax, dword [ram_address]
        mov dword [ehci_TD_buffer_pointer], eax
        mov eax, dword [sector_count]
        shl eax, 9
        mov dword [ehci_TD_buffer_size], eax
        mov dword [ehci_TD_packet_id], 1
        mov dword [ehci_TD_toggle_pointer], ehci_in_toggle
        mov eax, dword [ehci_in_endpoint]
        mov dword [ehci_TD_endpoint], eax
        call ehci_make_run_TD

    .pause_3:
        mov ecx, 01000000h
    
    .pause_4:
        sub ecx, 1
        jnz .pause_4

    .transport_status:
        mov dword [ehci_TD_buffer_pointer], ehci_scsi_csw
        mov dword [ehci_TD_buffer_size], 13
        mov dword [ehci_TD_packet_id], 1
        mov dword [ehci_TD_toggle_pointer], ehci_in_toggle
        mov eax, dword [ehci_in_endpoint]
        mov dword [ehci_TD_endpoint], eax
        call ehci_make_run_TD

    .pause_5:
        mov ecx, 01000000h
    
    .pause_6:
        sub ecx, 1
        jnz .pause_6

    .return:
        ret

    .request:
        dd 43425355h
        dd 0AABBCCDDh
        dd 0
        db 80h
        db 0
        db 10
        db 28h
        db 0
        dd 0
        dw 0
        db 0
        db 0
        times 6 db 0
