; ==============================================================================
; file: "souce/system/kernel/ext2.inc"
; use: to be included in "source/system/kernel.asm"
; ==============================================================================

ext2_compare_filename:
    
        pusha
    
    .check_next:
        mov al, byte [esi]
        cmp al, byte [edi]
        jne .return
        inc esi
        inc edi
        dec cl
        cmp cl, 0
        jne .check_next
   
    .return:
        popa
        ret

ext2_read_entry:

        pusha
        mov ebx, 0x0006C000 ; dword [ram_address_prev]
        ; mov esi, dword [string_1]
        
    .check_entry:
        mov edi, ebx
        add edi, 6
        mov cl, byte [esi]
        add cl, 2
        call ext2_compare_filename
        je .read
    
    .go_to_next:
        movzx eax, byte [ebx+6]
        mov edx, 0
        mov ecx, 4
        div ecx
        cmp edx, 0
        je .add_no_pad
    
    .add_pad:
        shl eax, 2
        add ebx, eax
        add ebx, 12
        jmp .check_entry
    
    .add_no_pad:
        shl eax, 2
        add ebx, eax
        add ebx, 8
        jmp .check_entry
    
    .read:
        mov eax, dword [ebx]
        mov dword [ext2_inode], eax
        call ext2_read_inode
    
    .return:
        popa
        ret
    
    .file_type db 0

ext2_read_file:

        mov eax, dword [ram_address]
        mov dword [.address], eax

    .read_superblock:
        mov dword [ram_address], 0x00060000
        mov dword [lba_sector], 2
        mov dword [sector_count], 2
        call ehci_read_sectors
    
    .process_block_size:
        mov cl, byte [ext2_block_size]
        mov dword [ext2_block_size], 2
        shl dword [ext2_block_size], cl
    
    .read_group_descriptor:
        cmp dword [ext2_block_size], 2
        je .read_sector_4
    
    .read_block_1:
        mov eax, dword [ext2_block_size]
        jmp .continue
    
    .read_sector_4:
        mov eax, 4
    
    .continue:
        mov dword [lba_sector], eax
        mov dword [sector_count], 8
        mov dword [ram_address], 0x00064000 ; group descriptor
        call ehci_read_sectors
    
    .read_inode_2:
        mov dword [ram_address], 0x0006C000 ; 0x00200000 ; root directory
        mov dword [ext2_inode], 2
        call ext2_read_inode
        
    .analyze_path:
        mov esi, dword [string_1]
        add esi, 5
    
    .copy_entry:
        add esi, 1
        mov edi, ext2_entry+2
        mov ecx, 0
    
    .copy_byte:
        mov al, byte [esi]
        cmp al, "/"
        je .set_dir
        cmp al, 0
        je .set_file
        mov byte [edi], al
        add esi, 1
        add edi, 1
        add ecx, 1
        jmp .copy_byte
    
    .set_dir:
        mov edi, ext2_entry
        mov byte [edi], cl
        mov byte [edi+1], 2
    
    .read_dir:
        push esi
        mov esi, edi
        call ext2_read_entry
        pop esi
        jmp .copy_entry
    
    .set_file:
        mov edi, ext2_entry
        mov byte [edi], cl
        mov byte [edi+1], 1
    
    .read_file:
        mov eax, dword [.address]
        mov dword [ram_address], eax
        push esi
        mov esi, edi
        call ext2_read_entry
        pop esi

    .return:
        ret

    .address dd 0

ext2_read_inode:
        
        dec dword [ext2_inode]
        mov dword [.direct_count], 0
        
        mov eax, dword [ram_address]
        mov dword [.current], eax
        
        ;cmp dword [ext2_inodes_per_group], 0
        ;je .return
    
    .get_group_number:
        mov eax, dword [ext2_inode]
        mov edx, 0
        div dword [ext2_inodes_per_group]
        mov dword [.group], eax
        mov dword [.index], edx
    
    .read_inode_table:
        shl eax, 5
        add eax, 0x00064000+8 ; from group descriptor
        mov ebx, eax
        mov eax, dword [ebx]
        imul eax, dword [ext2_block_size]
        mov dword [lba_sector], eax
        mov dword [sector_count], 8
        mov dword [ram_address], 0x00068000 ; to inode table
        call ehci_read_sectors
        
        ;mov eax, dword [.current]
        ;mov dword [ram_address], eax
    
    .read_inode_direct_block:
        mov ebx, dword [ext2_inode_struct_size]
        imul ebx, dword [.index]
        add ebx, 0x00068000+40 ; from inode table
        add ebx, dword [.direct_count]
        add ebx, dword [.direct_count]
        add ebx, dword [.direct_count]
        add ebx, dword [.direct_count]
        mov eax, dword [ebx]
        cmp eax, 0
        je .return
        imul eax, dword [ext2_block_size]
        mov dword [lba_sector], eax

        mov eax, dword [.direct_count]
        imul eax, dword [ext2_block_size]
        shl eax, 9
        add eax, dword [.current]
        mov dword [ram_address], eax
        
        mov eax, dword [ext2_block_size]
        mov dword [sector_count], eax
        
        call ehci_read_sectors
        
        add dword [.direct_count], 1
        cmp dword [.direct_count], 12
        jb .read_inode_direct_block
    
    .return:
        mov eax, dword [.current]
        mov dword [ram_address], eax
        inc dword [ext2_inode]
        ret
        
    .direct_count dd 0
    .group dd 0
    .index dd 0
    .current dd 0
    
    
ext2_total_inodes equ 0x00060000+0
ext2_total_blocks equ 0x00060000+4
ext2_block_size   equ 0x00060000+24
ext2_blocks_per_group equ 0x00060000+32
ext2_inodes_per_group equ 0x00060000+40
ext2_inode_struct_size equ 0x00060000+88

ext2_inode equ 0x00014000
ext2_entry equ 0x00014004
