; ==============================================================================
; file: "source/system/kernel/pci.inc"
; use: to be included in "source/system/kernel.asm"
; ==============================================================================

pci_read_dword:

; -----------------------------------------------------
; description: reads dword from pci configuration space
; input: dword [pci_address] = config address
;        dword [pci_offset] = offset register
; output: dword [pci_data] = data from port
; altered: {none}
; -----------------------------------------------------

    .send_command:
        pusha                           ; registers are preserved
        mov eax, dword [pci_address]    ; we get pci configuration address
        add eax, dword [pci_offset]     ; and we add offset to this address
        mov dx, 0CF8h                   ; then we send it to command port
        out dx, eax

    .read_data:
        mov dx, 0CFCh                   ; we read from data port
        in eax, dx                      ; a dword corresponding to that address
        mov dword [pci_data], eax

    .return:
        popa                            ; registers are restored
        ret

; ==============================================================================

pci_write_dword:

; ----------------------------------------------------
; description: writes dword to pci configuration space
; input: dword [pci_address] = config address
;        dword [pci_offset] = offset register
;        dword [pci_data] = data to port
; output: {none}
; altered: {none}
; ----------------------------------------------------

    .send_command:
        pusha                           ; registers are preserved
        mov eax, dword [pci_address]    ; we get pci configuration address
        add eax, dword [pci_offset]     ; and we add offset to this address
        mov dx, 0xCF8                   ; then we send to command port
        out dx, eax

    .send_dword_data:
        mov eax, dword [pci_data]       ; we write to data port
        mov dx, 0xCFC                   ; a dword corresponding to that address
        out dx, eax

    .return:
        popa                            ; registers are restored
        ret

; ==============================================================================

pci_set_config_address:

; --------------------------------------------------------
; description: sets configuration address
; input: dword [pci_bus] = pci address bus
;        dword [pci_dev] = pci address device
;        dword [pci_fun] = pci address funcion
; output: dword [pci_address] = pci config address enabled
; altered: {none}
; --------------------------------------------------------

    .prepare:
        pusha                           ; registers are preserved
        
    .get_bus:
        mov eax, dword [pci_bus]        ; we save bus to bits [23:16]
        shl eax, 16
        mov dword [pci_address], eax
        
    .get_dev:
        mov eax, dword [pci_dev]        ; we save dev to bits [15:11]
        shl eax, 11
        or dword [pci_address], eax
    
    .get_fun:
        mov eax, dword [pci_fun]        ; save fun to bits [10:8]
        shl eax, 8
        or dword [pci_address], eax
    
    .enable_address:
        or dword [pci_address], 0x80000000 ; address is enabled if bit 31 is set
    
    .return:
        popa                            ; registers are restored
        ret
